;        RM386 CLOAKING API
;
;Copyright (c) 1992, HELIX SOFTWARE COMPANY, Inc.
;
;

;------------------------------------------------------------------------------
;Transition to protected code loaded at xms handle (XMS CALL 82H)
;
;
;       Entry:  DX = XMS handle of block containing code
;               CL = 0 for 16 bit code, non zero for 32 bit code
;
;       Exit:   AX = 0 failure
;                    1 success
;                    BL = 0A2h invalid handle
;                    BL = 0B0h out of GDT selectors
;
;       protected mode routine at offset 0 in XMS block is called
;       with registers as follows:
;
;               EBX     physical address of start of block
;               CS      code selector for XMS block as 16 or 32 bit at EBX
;               DS      data selector for XMS block as Big starts at EBX
;               ES      selector for V86 memory acess to full 1 mb + 64K HMA
;               GS      selector for all memory mapped flat ie (phys=linear)
;               ESI     retained from V86 mode
;               EDI     retained from V86 mode
;
;------------------------------------------------------------------------------

;        PROTECTED MODE API AVAILABLE THROUGH PROTECTED INT 2CH
;
;Example:       allocate a GDT data selector for segment B000, limit 32k
;
;               MOV     EBX,0B0000H             ;Note B000 * 10H
;               MOV     EDX,32*1024-1
;               MOV     CX,92H                  ;Data, DPL 0, small
;               MOV     AX,ALLOC_GDT_SELECTOR
;               INT     2CH
;               JC      NO_MORE_SELECTORS
;               MOV     MY_B000_SELECTOR,AX
;

;------------------------------------------------------------------------------
;allocate a gdt selector
;
;       Entry:  EBX = selector base
;               EDX = selector limit
;               CL = selector access
;               CH = selector mode
;
;
;       Exit:   if no carry
;                       AX = selector to use
;               if carry
;                       AX = 1 -- out of selectors
;
;------------------------------------------------------------------------------

;------------------------------------------------------------------------------
;Free GDT selector
;
;       Entry:  SI = selector
;
;       Exit:   if no carry
;                       completed ok
;               if carry
;                       AX = 2 -- Invalid selector
;                          = 3 -- Selector not user selector
;                          = 4 -- Selector not allocated
;
;------------------------------------------------------------------------------

;------------------------------------------------------------------------------
;Set selector base
;
;       Entry:  SI = selector
;               EBX = physical start address
;
;       Exit:   if no carry
;                       completed ok
;               if carry
;                       AX = 2 -- Invalid selector
;                          = 3 -- Selector not user selector
;                          = 4 -- Selector not allocated
;
;------------------------------------------------------------------------------

;------------------------------------------------------------------------------
;Set selector access
;
;       Entry:  SI = selector
;               CL = R/W access bits for segment
;
;       Exit:   if no carry
;                       completed ok
;               if carry
;                       AX = 2 -- Invalid selector
;                          = 3 -- Selector not user selector
;                          = 4 -- Selector not allocated
;
;------------------------------------------------------------------------------

;------------------------------------------------------------------------------
;Set selector mode
;
;       Entry:  SI = selector
;               CL = granularity
;
;       Exit:   if no carry
;                       completed ok
;               if carry
;                       AX = 2 -- Invalid selector
;                          = 3 -- Selector not user selector
;                          = 4 -- Selector not allocated
;
;------------------------------------------------------------------------------

;------------------------------------------------------------------------------
;Set selector limit
;
;       Entry:  SI = selector
;               EBX = limit
;
;       Exit:   if no carry
;                       completed ok
;               if carry
;                       AX = 2 -- Invalid selector
;                          = 3 -- Selector not user selector
;                          = 4 -- Selector not allocated
;
;------------------------------------------------------------------------------

;------------------------------------------------------------------------------
;validate_selector
;
;       Entry:  SI = selector
;
;       Exit:   if no carry
;                       completed ok
;               if carry
;                       AX = 2 -- Invalid selector
;                          = 3 -- Selector not user selector
;                          = 4 -- Selector not allocated
;
;------------------------------------------------------------------------------

;------------------------------------------------------------------------------
;get IDT entry -- return selctor:offset of an IDT vector
;
;       Entry:  CL = Int number
;
;       Exit:   DX:EBX current far call address
;
;------------------------------------------------------------------------------

;------------------------------------------------------------------------------
;set IDT entry -- set an IDT vector to selctor:offset
;
;       Entry:  CL = Int number
;               DX:EBX current far call address
;
;       Exit:   none
;
;------------------------------------------------------------------------------

;------------------------------------------------------------------------------
;get_passalong -- return current value of PM pass-along api
;
;       Entry:  none
;
;       Exit:   DX:EBX current far call address
;
;------------------------------------------------------------------------------

;------------------------------------------------------------------------------
;set_passalong -- set PM pass-along api address (will be far called on int 2C
;                 from v86 mode)
;
;       Entry:  DX:EBX far call address to set
;
;       Exit:   none
;
;This is the preferred method for communicating with V86 mode TSR stubs.
;Passalong address is called on each V86 mode int 2C with registers as follows:
;       EAX     = CS:IP of int 2c as 32 bit dword
;       SS:EBX  = pointer to Caller registers (see caller_reg_struc below)
;       carry flag clear
;       others undefined
;
;On return to the passalong handler, if the carry is set RM386 will return
;immediately to V86 mode; If the carry is clear, RM386 will pass the INT 2Ch
;to the V86 mode int 2c handler as specified by the 2c int vector.
;
;------------------------------------------------------------------------------

;------------------------------------------------------------------------------
;get selector limit
;
;       Entry:  SI = selector
;
;       Exit:   if no carry
;                       EBX = limit
;                       completed ok
;               if carry
;                       AX = 2 -- Invalid selector
;                          = 3 -- Selector not user selector
;                          = 4 -- Selector not allocated
;
;------------------------------------------------------------------------------

;------------------------------------------------------------------------------
;get selector base
;
;       Entry:  SI = selector
;
;
;       Exit:   if no carry
;                       EBX = linear start address
;                       completed ok
;               if carry
;                       AX = 2 -- Invalid selector
;                          = 3 -- Selector not user selector
;                          = 4 -- Selector not allocated
;
;------------------------------------------------------------------------------

;------------------------------------------------------------------------------
;save protected state -- save current state of rm386 protect mode stuff
;
;       Entry:  ES:EDI = BUFFER FOR STATE
;
;       Exit:   BUFFER CONTAINS STATE
;------------------------------------------------------------------------------

;------------------------------------------------------------------------------
;restore protected state -- restore previously saved state
;
;       Entry:  DS:ESI = BUFFER FOR STATE
;
;       Exit:   if cy set
;                       Invalid buffer
;               else
;                       BUFFER RESTORED
;------------------------------------------------------------------------------

;------------------------------------------------------------------------------
;get protected state buffer size
;
;       Entry:  none
;
;       Exit:   eax = number of bytes needed to save protected state
;
;------------------------------------------------------------------------------

;------------------------------------------------------------------------------
;issue protected xms call -- call an xms function from protected mode
;
;       Entry:  not currently implemented
;
;       Exit:
;
;------------------------------------------------------------------------------

;------------------------------------------------------------------------------
;set v86 ss:sp -- set the ss:sp used in returning to v86 mode
;                 (be sure to save current protected state first, or not)
;
;       Entry:  dx:ebx = ss:esp for v86
;
;       Exit:   none
;
;It is far simpler to modify the Caller register structure directly.
;
;------------------------------------------------------------------------------

;
;       NOTE: the following three routines are slow as they require saving
;               the protected mode stack and state.  For faster operation
;               save your own internal state, modify the Caller Reg struc
;               to point to a routine in v86 mode that performs the desired
;               function and then issues another int 2ch. The Call v86 and
;               chain v86 routines should be used when timing is not critical


;------------------------------------------------------------------------------
;call v86 mode proc -- call a procedure running in v86 mode
;
;       Entry:  ds:ebx = client register structure for v86 mode call
;               note: uses v86 mode stack supplied in client structure
;               calls the routine at the cs:ip supplied in the client structure
;
;       Exit:   cy set = no more nested procedure call space available
;
;------------------------------------------------------------------------------

;------------------------------------------------------------------------------
;call v86 mode int -- issue a pseudo interrupt while in v86 mode
;
;       Entry:  ds:ebx = client register structure for v86 mode call
;               cx = interrupt number
;               note: uses v86 mode stack supplied in client structure
;
;       Exit:   cy set = no more nested procedure call space available
;
;------------------------------------------------------------------------------

;------------------------------------------------------------------------------
;chain to v86 mode int -- call an interrupt routine running in v86 mode
;
;       Entry:  ds:ebx = client register structure for v86 mode call
;               note: uses v86 mode stack supplied in client structure
;               performs pseudo of:
;                       push    Client_eflags
;                       push    ip
;                       push    cs
;                       jmp     Client_cs:Client_ip
;
;
;       Exit:   cy set = no more nested procedure call space available
;
;------------------------------------------------------------------------------



;------------------------------------------------------------------------------
;get rm tss esp - return esp0 value from tss
;
;       Entry:  none
;
;       Exit:   eax = tss esp0
;
;------------------------------------------------------------------------------

;------------------------------------------------------------------------------
;set secondary stack -- inform rm386 of location of ring 3 int stack
;
;       Entry:  ebx = ring 3 esp
;               dx = ring 3 ss
;
;       Exit:   none
;
;------------------------------------------------------------------------------

;------------------------------------------------------------------------------
;set hardware pic -- set the hardware base for the master and slave pic
;
;       Entry:  bl = master pic base
;               cl = slave pic base
;
;       Exit:   none
;
;------------------------------------------------------------------------------

;------------------------------------------------------------------------------
;protected_VDS_call -- call VDS from protected mode
;
;       Entry:  CX = 81xx for vds function dispatcher
;               other regs set up for call
;
;       Exit:   per vds cy set if error
;
;------------------------------------------------------------------------------

;------------------------------------------------------------------------------
;get_port_passalong -- get current i/o port passalong address
;
;       Entry:  none
;
;       Exit:   DX:EBX = current i/o trapping passalong address
;
;------------------------------------------------------------------------------

;------------------------------------------------------------------------------
;set_port_passalong -- set current i/o port passalong address
;
;       Entry:  DX:EBX = new i/o trapping passalong address to set
;
;       Exit:   none
;
;Passalong address is called when a port i/o op is trapped to a port that is
;not reserved for RM386 use (ie not a DMA port.)  The passalong address is
;called with the registers set as follows:
;       EAX    = CS:IP of address of faulting instruction (not valid if were
;                executing in protected mode ring 3)
;       SS:EBX = Caller register structure -- may be a RING 3 type struct
;                so need to check EFLAGS for V86 mode bit
;       CX     = First two bytes of I/O operation instruction
;       DX     = Port to which I/O is being performed.
;       Carry is clear
;
;If pass along returns with carry clear the I/O operation is performed by
;RM386 if carry is set the I/O is not performed.  In either case the faulting
;I/O operation is skipped by RM386 and should not be skipped by the passalong
;handler--do not modify Client CS:EIP.
;
;------------------------------------------------------------------------------


;------------------------------------------------------------------------------
;trap_io_port -- set the trap bit in the tss for specified port
;
;       Entry:  DX port to trap
;
;       Exit:   cy if io port is out of range or reserved
;               (out of range if port is above 0FFFh and NOHIGH_IO is set
;                reserved if required by RM386 eg: DMA, etc.)
;
;------------------------------------------------------------------------------

;------------------------------------------------------------------------------
;untrap_io_port -- reset the trap bit in the tss for specified port
;
;       Entry:  DX port to untrap
;
;       Exit:   cy if io port is out of range or reserved
;
;------------------------------------------------------------------------------

;------------------------------------------------------------------------------
;get_port_state -- return the current trap bit setting for specified port
;
;       Entry:  DX port to examine
;
;       Exit:   cy if io port is out of range or reserved
;               BX=0 if port not trapped, 1 if port trapped
;
;------------------------------------------------------------------------------


;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------


;
;EQUATES FOR USE WITH RM386 PROTECTED API
;
alloc_GDT_selector      EQU     0               ;*
free_GDT_selector       EQU     1               ;*
set_selector_base       EQU     2               ;*
set_selector_limit      EQU     3               ;*
set_selector_access     EQU     4               ;*
set_selector_mode       EQU     5               ;*
get_IDT_entry           EQU     6               ;*
set_IDT_entry           EQU     7               ;*
get_passalong           EQU     8
set_passalong           EQU     9
get_selector_base       EQU     0ah             ;*
get_selector_limit      EQU     0bh             ;*
;
;FUNCTIONS C THROUGH 1F RESERVED FOR CLOAKED BIOS USE UNDER WINDOWS
;
get_ps_buffer_size      EQU     20h
save_protected_state    EQU     21h
restore_protected_state EQU     22h
issue_protected_xms     EQU     23h
set_v86_sssp            EQU     24h
call_v86_proc           EQU     25h
call_v86_int            EQU     26h
chain_to_v86_int        EQU     27H
get_rm_tss_esp          EQU     28h
set_secondary_stack     EQU     29h
set_hardware_pic        EQU     2ah
protected_VDS_call      EQU     2bh
get_port_passalong      EQU     2ch
set_port_passalong      EQU     2dh
trap_io_port            EQU     2eh
untrap_io_port          EQU     2fh
get_port_state          EQU     30h
;
;FUNCTIONS LABELED WITH * ARE ALSO PROVIDED BY RMLODHI.VXD UNDER WINDOWS
;OTHERS WILL BE MADE AVAILABLE UNDER WINDOWS IN FUTURE RELEASES
;
;------------------------------------------------------------------------------
;
;       CALLER REGISTER STRUCTURE
;
CALLER_REG_STRUC STRUC
CALLER_EDI      dd      ?               ; EDI
CALLER_ESI      dd      ?               ; ESI
CALLER_EBP      dd      ?               ; EBP
                dd      ?               ;  ESP AT PUSHAD--DO NOT USE
CALLER_EBX      dd      ?               ; EBX
CALLER_EDX      dd      ?               ; EDX
CALLER_ECX      dd      ?               ; ECX
CALLER_EAX      dd      ?               ; EAX
CALLER_Error    dd      ?               ; ERROR CODE (USUALLY NOTHING)
CALLER_EIP      dd      ?               ; EIP
CALLER_CS       dw      ?               ; CS
                dw      ?               ;
CALLER_EFlags   dd      ?               ; EFLAGS
CALLER_ESP      dd      ?               ; ESP
CALLER_SS       dw      ?               ; SS
                dw      ?               ;
CALLER_ES       dw      ?               ; ES ;this and following not available
                dw      ?               ;    ;if this structure is R3
CALLER_DS       dw      ?               ; DS ;Ring 3 exceptions can be detected
                dw      ?               ;    ;using the VM bit in Eflags
CALLER_FS       dw      ?               ; FS
                dw      ?               ;
CALLER_GS       dw      ?               ; GS
                dw      ?               ;
CALLER_REG_STRUC ENDS
