// wuidbint.c - WUIMAN DataBase INTerface definitions
// Copyright (c) 1993 by Ron Burk

#include <string.h>

#include "wuistd.h"
#include "wuidbint.h"

#if !(defined(_INC_WINDOWS) || defined(__WINDOWS_H))
    #include <windows.h>
#endif

const char *SENTINEL = "$";

// constructor - just save database name (assume it's legal)
TWuiDatabase::TWuiDatabase(const char *Database)
    :   DatabaseName_(StringClone(Database))
    {   MEMBERASSERT();
    }

// destructor - free up database name
TWuiDatabase::~TWuiDatabase()
    {   MEMBERASSERT();
    if(DatabaseName_)
        delete[] DatabaseName_;
    }

// WriteItem - Set value for item under specified path.
int     TWuiDatabase::WriteItem(const char *Path,
                              const char *Key, char *Buffer)
    {   MEMBERASSERT();
    char   *Copy = new char[strlen(Buffer)+2];
    strcpy(Copy, Buffer);
    strcat(Copy, SENTINEL);
    int Status = WritePrivateProfileString(Path, Key, Copy,
                                        DatabaseName_) != 0;
    delete[] Copy;
    return Status;
    }

// ReadItem - Get value for named item under specified path.
int     TWuiDatabase::ReadItem(const char *Path,
        const char *Key, char *Buffer, size_t Length)
    {   MEMBERASSERT();
    char   *Copy = new char[Length+2];
    int Status = GetPrivateProfileString(Path, Key, "",
                               Copy, Length, DatabaseName_);
    if(Status <= 0)
        Status  = -1;
    else
        {
        ASSERT(Copy[strlen(Copy)-1] == SENTINEL[0]);
        Status          = strlen(Copy)-1;
        Copy[Status]    = '\0'; // trim sentinel byte before returning
        strcpy(Buffer, Copy);
        }
    delete[] Copy;
    return Status;
    }

// DeleteItem - Delete named item under specified path.
int     TWuiDatabase::DeleteItem(const char *Path,
                                            const char *Key)
    {   MEMBERASSERT();
    return WritePrivateProfileString(Path, Key, NULL,
                                        DatabaseName_) != 0;
    }

// DeletePath - Delete path itself (and any children)
int     TWuiDatabase::DeletePath(const char *Path)
    {   MEMBERASSERT();
    return WritePrivateProfileString(Path, NULL, NULL,
                                        DatabaseName_) != 0;
    }

// GetKeys - Get the children/attribute of specified path.
TWuiDatabaseKeys *TWuiDatabase::GetKeys(const char *Path)
    {   MEMBERASSERT();
    const int MAXKEYS   = (1024*4);
    char *Keys          = new char[MAXKEYS];

    // if no keys found, or too many to fit in buffer,
    // then fail by returning 0.

    int Status  = ReadItem(Path, NULL, Keys, MAXKEYS);
    if(Status <= 0 || (Status >= MAXKEYS-1))
        {
        delete[] Keys;
        return 0;
        }
    else
        return new TWuiDatabaseKeys(Keys);
    }

// constructor - Save string of keys.
TWuiDatabaseKeys::TWuiDatabaseKeys(char *Keys)
    :   Keys_(Keys), Current_(Keys)
    {   MEMBERASSERT();
    ASSERT(Keys != 0);
    }

// destructor - Free string of keys.
TWuiDatabaseKeys::~TWuiDatabaseKeys()
    {   MEMBERASSERT();
    if(Keys_)
        delete[] Keys_;
    }

// FirstKey - fetch first key of series.
const char *TWuiDatabaseKeys::FirstKey()
    {   MEMBERASSERT();
    return Current_ = Keys_;
    }

// NextKey - fetch next key in series, NULL if at end.
const char *TWuiDatabaseKeys::NextKey()
    {   MEMBERASSERT();
    ASSERT(Current_ != NULL);
    if(*Current_)
        Current_    += strlen(Current_)+1;
    return Current_;
    }

